@@ -49,14 +49,14 @@ public:
     ResultsStream *search(const AbstractResourcesBackend::Filters &search) override;
     ResultsStream *findResourceByPackageName(const QUrl &search);
     bool isValid() const override;
+    int fetchingUpdatesProgress() const override
+    {
+        return m_isFetching > 0 ? 42 : 100;
+    }
 
     Transaction *installApplication(AbstractResource *app) override;
     Transaction *installApplication(AbstractResource *app, const AddonList &addons) override;
     Transaction *removeApplication(AbstractResource *app) override;
-    bool isFetching() const override
-    {
-        return m_isFetching > 0;
-    }
     void checkForUpdates() override;
     QString displayName() const override;
     bool hasApplications() const override
@@ -111,7 +111,6 @@ private:
         return m_installations.constFirst();
     }
     QSharedPointer<FlatpakSource> integrateRemote(FlatpakInstallation *flatpakInstallation, FlatpakRemote *remote);
-    FlatpakRemote *getFlatpakRemoteByUrl(const QString &url, FlatpakInstallation *installation) const;
     FlatpakResource *getRuntimeForApp(FlatpakResource *resource) const;
     FlatpakResource *resourceForComponent(const AppStream::Component &component, const QSharedPointer<FlatpakSource> &source) const;
     void checkRepositories(const QMap<QString, QStringList> &repositories);
@@ -127,6 +126,7 @@ private:
     bool updateAppSizeFromRemote(FlatpakResource *resource);
     void updateAppState(FlatpakResource *resource);
     QSharedPointer<FlatpakSource> findSource(FlatpakInstallation *installation, const QString &origin) const;
+    QSharedPointer<FlatpakSource> findSourceUrl(FlatpakInstallation *installation, const QString &url) const;
 
     QVector<StreamResult> resultsByAppstreamName(const QString &name) const;
     void acquireFetching(bool f);
@@ -135,6 +135,7 @@ private:
     FlatpakRemote *installSource(FlatpakResource *resource);
 
     ResultsStream *deferredResultStream(const QString &streamName, std::function<QCoro::Task<>(ResultsStream *)> callback);
+    ResultsStream *deferredResultStreamNoFinish(const QString &streamName, std::function<QCoro::Task<>(ResultsStream *)> callback);
     // Returned Installation and Ref objects are g_object_ref'ed, caller is responsible for calling g_object_unref.
     QCoro::Task<QHash<FlatpakInstallation *, QList<FlatpakInstalledRef *>>> listInstalledRefsForUpdate();
 
