@@ -33,10 +33,6 @@ class RpmOstreeBackend : public AbstractResourcesBackend
 public:
     explicit RpmOstreeBackend(QObject *parent = nullptr);
 
-    /* Convenience function to set the fetching status and emit the
-     * corresponding signal */
-    void setFetching(bool);
-
     int updatesCount() const override;
     AbstractBackendUpdater *backendUpdater() const override;
     AbstractReviewsBackend *reviewsBackend() const override;
@@ -49,7 +45,8 @@ public:
     Transaction *installApplication(AbstractResource *, const AddonList &) override;
     Transaction *removeApplication(AbstractResource *) override;
 
-    bool isFetching() const override;
+    int fetchingUpdatesProgress() const override;
+
     void checkForUpdates() override;
     QString displayName() const override;
     bool hasApplications() const override;
@@ -80,7 +77,7 @@ private:
     void setupTransaction(RpmOstreeTransaction::Operation op, QString arg = {});
 
     /* Called when a new major version is found to setup user facing actions */
-    void foundNewMajorVersion(const QString &newMajorVersion);
+    void foundNewMajorVersion(const AppStream::Release &release);
 
     /* Set to true once we've successfully registrered with rpm-ostree */
     bool m_registrered;
